@@ -71,12 +71,12 @@ class DirDef : public Definition
 
     // generate output
     void writeDocumentation(OutputList &ol);
-    void writeDepGraph(FTextStream &t);
     void writeTagFile(FTextStream &t);
 
     static DirDef *mergeDirectoryInTree(const QCString &path);
     bool visited;
     void setDiskName(const QCString &name) { m_diskName = name; }
+    void sort();
 
   private:
     friend void computeDirDependencies();
@@ -139,6 +139,7 @@ class UsedDir
     const FilePairDict &filePairs() const { return m_filePairs; }
     const DirDef *dir() const { return m_dir; }
     bool inherited() const { return m_inherited; }
+    void sort();
 
   private:
     DirDef *m_dir;
