@@ -42,7 +42,7 @@ static const char types[][NUM_HTML_LIST_TYPES] = {"1", "a", "i", "A"};
 static QCString convertIndexWordToAnchor(const QString &word)
 {
   static char hex[] = "0123456789abcdef";
-  QCString result;
+  QCString result="a";
   const char *str = word.data();
   unsigned char c;
   if (str)
@@ -50,12 +50,11 @@ static QCString convertIndexWordToAnchor(const QString &word)
     while ((c = *str++))
     {
       if ((c >= 'a' && c <= 'z') || // ALPHA
-          (c >= 'A' && c <= 'A') || // ALPHA
+          (c >= 'A' && c <= 'Z') || // ALPHA
           (c >= '0' && c <= '9') || // DIGIT
           c == '-' ||
           c == '.' ||
-          c == '_' ||
-          c == '~'
+          c == '_'
          )
       {
         result += c;
@@ -63,7 +62,7 @@ static QCString convertIndexWordToAnchor(const QString &word)
       else
       {
         char enc[4];
-        enc[0] = '%';
+        enc[0] = ':';
         enc[1] = hex[(c & 0xf0) >> 4];
         enc[2] = hex[c & 0xf];
         enc[3] = 0;
@@ -410,7 +409,7 @@ void HtmlDocVisitor::visit(DocVerbatim *s)
 
         forceEndParagraph(s);
         fileName.sprintf("%s%d%s", 
-            (Config_getString("HTML_OUTPUT")+"/inline_dotgraph_").data(), 
+            (Config_getString(HTML_OUTPUT)+"/inline_dotgraph_").data(), 
             dotindex++,
             ".dot"
            );
@@ -424,14 +423,14 @@ void HtmlDocVisitor::visit(DocVerbatim *s)
           file.writeBlock( s->text(), s->text().length() );
           file.close();
 
-          m_t << "<div align=\"center\">" << endl;
+          m_t << "<div class=\"dotgraph\">" << endl;
           writeDotFile(fileName,s->relPath(),s->context());
           visitPreCaption(m_t, s);
           visitCaption(this, s->children());
           visitPostCaption(m_t, s);
           m_t << "</div>" << endl;
 
-          if (Config_getBool("DOT_CLEANUP")) file.remove();
+          if (Config_getBool(DOT_CLEANUP)) file.remove();
         }
         forceStartParagraph(s);
       }
@@ -444,7 +443,7 @@ void HtmlDocVisitor::visit(DocVerbatim *s)
         QCString baseName(4096);
 
         baseName.sprintf("%s%d", 
-            (Config_getString("HTML_OUTPUT")+"/inline_mscgraph_").data(), 
+            (Config_getString(HTML_OUTPUT)+"/inline_mscgraph_").data(), 
             mscindex++
             );
         QFile file(baseName+".msc");
@@ -461,14 +460,14 @@ void HtmlDocVisitor::visit(DocVerbatim *s)
           file.writeBlock( text, text.length() );
           file.close();
 
-          m_t << "<div align=\"center\">" << endl;
+          m_t << "<div class=\"mscgraph\">" << endl;
           writeMscFile(baseName+".msc",s->relPath(),s->context());
           visitPreCaption(m_t, s);
           visitCaption(this, s->children());
           visitPostCaption(m_t, s);
           m_t << "</div>" << endl;
 
-          if (Config_getBool("DOT_CLEANUP")) file.remove();
+          if (Config_getBool(DOT_CLEANUP)) file.remove();
         }
         forceStartParagraph(s);
       }
@@ -477,9 +476,9 @@ void HtmlDocVisitor::visit(DocVerbatim *s)
       {
         forceEndParagraph(s);
 
-        static QCString htmlOutput = Config_getString("HTML_OUTPUT");
+        static QCString htmlOutput = Config_getString(HTML_OUTPUT);
         QCString baseName = writePlantUMLSource(htmlOutput,s->exampleFile(),s->text());
-        m_t << "<div align=\"center\">" << endl;
+        m_t << "<div class=\"plantumlgraph\">" << endl;
         writePlantUMLFile(baseName,s->relPath(),s->context());
         visitPreCaption(m_t, s);
         visitCaption(this, s->children());
@@ -579,6 +578,31 @@ void HtmlDocVisitor::visit(DocInclude *inc)
                                            -1,    // endLine
                                            TRUE,  // inlineFragment
                                            0,     // memberDef
+                                           FALSE, // show line number
+                                           m_ctx  // search context
+                                          );
+         m_t << PREFRAG_END;
+         forceStartParagraph(inc);
+      }
+      break;
+    case DocInclude::SnipWithLines:
+      {
+         forceEndParagraph(inc);
+         m_t << PREFRAG_START;
+         QFileInfo cfi( inc->file() );
+         FileDef fd( cfi.dirPath().utf8(), cfi.fileName().utf8() );
+         Doxygen::parserManager->getParser(inc->extension())
+                               ->parseCode(m_ci,
+                                           inc->context(),
+                                           extractBlock(inc->text(),inc->blockId()),
+                                           langExt,
+                                           inc->isExample(),
+                                           inc->exampleFile(), 
+                                           &fd,
+                                           lineBlock(inc->text(),inc->blockId()),
+                                           -1,    // endLine
+                                           FALSE, // inlineFragment
+                                           0,     // memberDef
                                            TRUE,  // show line number
                                            m_ctx  // search context
                                           );
@@ -586,6 +610,11 @@ void HtmlDocVisitor::visit(DocInclude *inc)
          forceStartParagraph(inc);
       }
       break;
+    case DocInclude::SnippetDoc: 
+    case DocInclude::IncludeDoc: 
+      err("Internal inconsistency: found switch SnippetDoc / IncludeDoc in file: %s"
+          "Please create a bug report\n",__FILE__);
+      break;
   }
 }
 
@@ -646,7 +675,7 @@ void HtmlDocVisitor::visit(DocFormula *f)
     m_t << "<p class=\"formulaDsp\">" << endl;
   }
 
-  if (Config_getBool("USE_MATHJAX"))
+  if (Config_getBool(USE_MATHJAX))
   {
     QCString text = f->text();
     bool closeInline = FALSE;
@@ -1037,7 +1066,7 @@ void HtmlDocVisitor::visitPre(DocPara *p)
 void HtmlDocVisitor::visitPost(DocPara *p)
 {
   bool needsTag = FALSE;
-  if (p && p->parent()) 
+  if (p->parent()) 
   {
     switch (p->parent()->kind()) 
     {
@@ -1067,7 +1096,7 @@ void HtmlDocVisitor::visitPost(DocPara *p)
   // the paragraph (<ul>,<dl>,<table>) then that will already have ended the 
   // paragraph and we don't need to do it here
   int nodeIndex = p->children().count()-1;
-  if (p && nodeIndex>=0)
+  if (nodeIndex>=0)
   {
     while (nodeIndex>=0 && p->children().at(nodeIndex)->kind()==DocNode::Kind_WhiteSpace)
     {
@@ -1301,6 +1330,11 @@ void HtmlDocVisitor::visitPre(DocHtmlTable *t)
 
   forceEndParagraph(t);
 
+  if (t->hasCaption())
+  {
+    m_t << "<a class=\"anchor\" id=\"" << t->caption()->anchor() << "\"></a>\n";
+  }
+
   QString attrs = htmlAttribsToString(t->attribs());
   if (attrs.isEmpty())
   {
@@ -1308,7 +1342,7 @@ void HtmlDocVisitor::visitPre(DocHtmlTable *t)
   }
   else
   {
-    m_t << "<table " << htmlAttribsToString(t->attribs()) << ">\n";
+    m_t << "<table" << htmlAttribsToString(t->attribs()) << ">\n";
   }
 }
 
@@ -1353,16 +1387,7 @@ void HtmlDocVisitor::visitPost(DocHtmlCell *c)
 void HtmlDocVisitor::visitPre(DocHtmlCaption *c)
 {
   if (m_hide) return;
-  bool hasAlign      = FALSE;
-  HtmlAttribListIterator li(c->attribs());
-  HtmlAttrib *att;
-  for (li.toFirst();(att=li.current());++li)
-  {
-    if (att->name=="align") hasAlign=TRUE;
-  }
-  m_t << "<caption" << htmlAttribsToString(c->attribs());
-  if (!hasAlign) m_t << " align=\"bottom\"";
-  m_t << ">";
+  m_t << "<caption" << htmlAttribsToString(c->attribs()) << ">";
 }
 
 void HtmlDocVisitor::visitPost(DocHtmlCaption *) 
@@ -1434,17 +1459,43 @@ void HtmlDocVisitor::visitPre(DocImage *img)
     }
     m_t << "<div class=\"image\">" << endl;
     QCString url = img->url();
+    QCString sizeAttribs;
+    if (!img->width().isEmpty())
+    {
+      sizeAttribs+=" width=\""+img->width()+"\"";
+    }
+    if (!img->height().isEmpty())
+    {
+      sizeAttribs+=" height=\""+img->height()+"\"";
+    }
     if (url.isEmpty())
     {
-      m_t << "<img src=\"" << img->relPath() << img->name() << "\" alt=\"" 
-          << baseName << "\"" << htmlAttribsToString(img->attribs()) 
-          << "/>" << endl;
+      if (img->name().right(4)==".svg")
+      {
+        m_t << "<object type=\"image/svg+xml\" data=\"" << img->relPath() << img->name()
+            << "\"" << sizeAttribs << htmlAttribsToString(img->attribs()) << ">" << baseName
+            << "</object>" << endl;
+      }
+      else
+      {
+        m_t << "<img src=\"" << img->relPath() << img->name() << "\" alt=\""
+            << baseName << "\"" << sizeAttribs << htmlAttribsToString(img->attribs())
+            << "/>" << endl;
+      }
     }
     else
     {
-      m_t << "<img src=\"" << correctURL(url,img->relPath()) << "\" " 
-          << htmlAttribsToString(img->attribs())
-          << "/>" << endl;
+      if (url.right(4)==".svg")
+      {
+        m_t << "<object type=\"image/svg+xml\" data=\"" << correctURL(url,img->relPath())
+            << "\"" << sizeAttribs << htmlAttribsToString(img->attribs()) << "></object>" << endl;
+      }
+      else
+      {
+        m_t << "<img src=\"" << correctURL(url,img->relPath()) << "\""
+            << sizeAttribs << htmlAttribsToString(img->attribs())
+            << "/>" << endl;
+      }
     }
     if (img->hasCaption())
     {
@@ -1605,7 +1656,7 @@ void HtmlDocVisitor::visitPost(DocSecRefList *s)
 
 //void HtmlDocVisitor::visitPre(DocLanguage *l)
 //{
-//  QString langId = Config_getEnum("OUTPUT_LANGUAGE");
+//  QString langId = Config_getEnum(OUTPUT_LANGUAGE);
 //  if (l->id().lower()!=langId.lower())
 //  {
 //    pushEnabled();
@@ -1615,7 +1666,7 @@ void HtmlDocVisitor::visitPost(DocSecRefList *s)
 //
 //void HtmlDocVisitor::visitPost(DocLanguage *l) 
 //{
-//  QString langId = Config_getEnum("OUTPUT_LANGUAGE");
+//  QString langId = Config_getEnum(OUTPUT_LANGUAGE);
 //  if (l->id().lower()!=langId.lower())
 //  {
 //    popEnabled();
@@ -1808,7 +1859,6 @@ void HtmlDocVisitor::visitPre(DocHtmlBlockQuote *b)
 {
   if (m_hide) return;
   forceEndParagraph(b);
-
   QString attrs = htmlAttribsToString(b->attribs());
   if (attrs.isEmpty())
   {
@@ -1816,7 +1866,7 @@ void HtmlDocVisitor::visitPre(DocHtmlBlockQuote *b)
   }
   else
   {
-    m_t << "<blockquote " << htmlAttribsToString(b->attribs()) << ">\n";
+    m_t << "<blockquote" << htmlAttribsToString(b->attribs()) << ">\n";
   }
 }
 
@@ -1902,24 +1952,8 @@ void HtmlDocVisitor::filterQuotedCdataAttr(const char* str)
     {
       case '&':  m_t << "&amp;"; break;
       case '"':  m_t << "&quot;"; break;
-       // For SGML compliance, and given the SGML declaration for HTML syntax,
-       // it's enough to replace these two, provided that the declaration
-       // for the HTML version we generate (and as supported by the browser)
-       // specifies that all the other symbols used in rawVal are
-       // within the right character class (i.e., they're not
-       // some multinational weird characters not in the BASESET).
-       // We assume that 1) the browser will support whatever is remaining
-       // in the formula and 2) the TeX formulae are generally governed
-       // by even stricter character restrictions so it should be enough.
-       //
-       // On some incompliant browsers, additional translation of
-       // '>' and '<' into "&gt;" and "&lt;", respectively, might be needed;
-       // but I'm unaware of particular modern (last 4 years) versions
-       // with such problems, so let's not do it for performance.
-       // Also, some brousers will (wrongly) not process the entity references
-       // inside the attribute value and show the &...; form instead,  
-       // so we won't create entites unless necessary to minimize clutter there.
-       // --vassilii 
+      case '<':  m_t << "&lt;"; break;
+      case '>':  m_t << "&gt;"; break;
       default:   m_t << c;
     }
   }
@@ -1944,7 +1978,7 @@ void HtmlDocVisitor::startLink(const QCString &ref,const QCString &file,
   if (!file.isEmpty()) m_t << file << Doxygen::htmlFileExtension;
   if (!anchor.isEmpty()) m_t << "#" << anchor;
   m_t << "\"";
-  if (!tooltip.isEmpty()) m_t << " title=\"" << substitute(tooltip,"\"","&quot;") << "\"";
+  if (!tooltip.isEmpty()) m_t << " title=\"" << convertToHtml(tooltip) << "\"";
   m_t << ">";
 }
 
@@ -1980,7 +2014,7 @@ void HtmlDocVisitor::writeDotFile(const QCString &fn,const QCString &relPath,
     baseName=baseName.left(i);
   }
   baseName.prepend("dot_");
-  QCString outDir = Config_getString("HTML_OUTPUT");
+  QCString outDir = Config_getString(HTML_OUTPUT);
   writeDotGraphFromFile(fn,outDir,baseName,GOF_BITMAP);
   writeDotImageMapFromFile(m_t,fn,outDir,relPath,baseName,context);
 }
@@ -2000,7 +2034,7 @@ void HtmlDocVisitor::writeMscFile(const QCString &fileName,
     baseName=baseName.left(i);
   }
   baseName.prepend("msc_");
-  QCString outDir = Config_getString("HTML_OUTPUT");
+  QCString outDir = Config_getString(HTML_OUTPUT);
   QCString imgExt = getDotImageExtension();
   MscOutputFormat mscFormat = MSC_BITMAP;
   if ("svg" == imgExt)
@@ -2024,7 +2058,7 @@ void HtmlDocVisitor::writeDiaFile(const QCString &fileName,
     baseName=baseName.left(i);
   }
   baseName.prepend("dia_");
-  QCString outDir = Config_getString("HTML_OUTPUT");
+  QCString outDir = Config_getString(HTML_OUTPUT);
   writeDiaGraphFromFile(fileName,outDir,baseName,DIA_BITMAP);
 
   m_t << "<img src=\"" << relPath << baseName << ".png" << "\" />" << endl;
@@ -2044,7 +2078,7 @@ void HtmlDocVisitor::writePlantUMLFile(const QCString &fileName,
   {
     baseName=baseName.left(i);
   }
-  static QCString outDir = Config_getString("HTML_OUTPUT");
+  static QCString outDir = Config_getString(HTML_OUTPUT);
   QCString imgExt = getDotImageExtension();
   if (imgExt=="svg")
   {
@@ -2061,6 +2095,42 @@ void HtmlDocVisitor::writePlantUMLFile(const QCString &fileName,
   }
 }
 
+/** Returns TRUE if the child nodes in paragraph \a para until \a nodeIndex
+    contain a style change node that is still active and that style change is one that
+    must be located outside of a paragraph, i.e. it is a center, div, or pre tag.
+    See also bug746162.
+ */
+static bool insideStyleChangeThatIsOutsideParagraph(DocPara *para,int nodeIndex)
+{
+  //printf("insideStyleChangeThatIsOutputParagraph(index=%d)\n",nodeIndex);
+  int styleMask=0;
+  bool styleOutsideParagraph=FALSE;
+  while (nodeIndex>=0 && !styleOutsideParagraph)
+  {
+    DocNode *n = para->children().at(nodeIndex);
+    if (n->kind()==DocNode::Kind_StyleChange)
+    {
+      DocStyleChange *sc = (DocStyleChange*)n;
+      if (!sc->enable()) // remember styles that has been closed already
+      {
+        styleMask|=(int)sc->style();
+      }
+      bool paraStyle = sc->style()==DocStyleChange::Center ||
+                       sc->style()==DocStyleChange::Div    ||
+                       sc->style()==DocStyleChange::Preformatted;
+      //printf("Found style change %s enabled=%d\n",sc->styleString(),sc->enable());
+      if (sc->enable() && (styleMask&(int)sc->style())==0 && // style change that is still active
+          paraStyle
+         )
+      {
+        styleOutsideParagraph=TRUE;
+      }
+    }
+    nodeIndex--;
+  }
+  return styleOutsideParagraph;
+}
+
 /** Used for items found inside a paragraph, which due to XHTML restrictions
  *  have to be outside of the paragraph. This method will forcefully end
  *  the current paragraph and forceStartParagraph() will restart it.
@@ -2074,7 +2144,7 @@ void HtmlDocVisitor::forceEndParagraph(DocNode *n)
     int nodeIndex = para->children().findRef(n);
     nodeIndex--;
     if (nodeIndex<0) return; // first node
-    while (nodeIndex>=0 && 
+    while (nodeIndex>=0 &&
            para->children().at(nodeIndex)->kind()==DocNode::Kind_WhiteSpace
           )
     {
@@ -2086,12 +2156,14 @@ void HtmlDocVisitor::forceEndParagraph(DocNode *n)
       //printf("n=%p kind=%d outside=%d\n",n,n->kind(),mustBeOutsideParagraph(n));
       if (mustBeOutsideParagraph(n)) return;
     }
-
+    nodeIndex--;
+    bool styleOutsideParagraph=insideStyleChangeThatIsOutsideParagraph(para,nodeIndex);
     bool isFirst;
     bool isLast;
     getParagraphContext(para,isFirst,isLast);
-    //printf("forceEnd first=%d last=%d\n",isFirst,isLast);
+    //printf("forceEnd first=%d last=%d styleOutsideParagraph=%d\n",isFirst,isLast,styleOutsideParagraph);
     if (isFirst && isLast) return;
+    if (styleOutsideParagraph) return;
 
     m_t << "</p>";
   }
@@ -2109,9 +2181,11 @@ void HtmlDocVisitor::forceStartParagraph(DocNode *n)
     DocPara *para = (DocPara*)n->parent();
     int nodeIndex = para->children().findRef(n);
     int numNodes  = para->children().count();
+    bool styleOutsideParagraph=insideStyleChangeThatIsOutsideParagraph(para,nodeIndex);
+    if (styleOutsideParagraph) return;
     nodeIndex++;
     if (nodeIndex==numNodes) return; // last node
-    while (nodeIndex<numNodes && 
+    while (nodeIndex<numNodes &&
            para->children().at(nodeIndex)->kind()==DocNode::Kind_WhiteSpace
           )
     {
