@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: membergroup.h,v 1.13 2001/03/19 19:27:41 root Exp $
+ * 
  *
- * Copyright (C) 1997-2005 by Dimitri van Heesch.
+ * Copyright (C) 1997-2015 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -18,44 +18,57 @@
 #ifndef MEMBERGROUP_H
 #define MEMBERGROUP_H
 
-#include "qtbc.h"
 #include <qlist.h>
 #include "sortdict.h"
+#include "types.h"
 
 #define DOX_NOGROUP -1
 
+class MemberList;
 class MemberDef;
 class ClassDef;
 class NamespaceDef;
 class FileDef;
-class MemberList;
 class GroupDef;
 class OutputList;
 class Definition;
+class StorageIntf;
+class FTextStream;
+struct ListItemInfo;
 
+/** A class representing a group of members. */
 class MemberGroup 
 {
   public:
+    MemberGroup();
     MemberGroup(Definition *parent,int id,const char *header,
-                const char *docs,const char *docFile);
+                const char *docs,const char *docFile,int docLine);
    ~MemberGroup();
     QCString header() const { return grpHeader; }
     int groupId() const { return grpId; }
     void insertMember(MemberDef *md);
-    void setAnchors(ClassDef *);
+    void setAnchors();
     void writePlainDeclarations(OutputList &ol,
-               ClassDef *cd,NamespaceDef *nd,FileDef *fd,GroupDef *gd);
+               ClassDef *cd,NamespaceDef *nd,FileDef *fd,GroupDef *gd,
+               ClassDef *inheritedFrom,const char *inheritId);
     void writeDeclarations(OutputList &ol,
-               ClassDef *cd,NamespaceDef *nd,FileDef *fd,GroupDef *gd);
-
+               ClassDef *cd,NamespaceDef *nd,FileDef *fd,GroupDef *gd,
+               bool showInline=FALSE);
     void writeDocumentation(OutputList &ol,const char *scopeName,
-               Definition *container);
+               Definition *container,bool showEnumValues,bool showInline);
     void writeDocumentationPage(OutputList &ol,const char *scopeName,
                Definition *container);
-    QCString documentation() { return doc; }
-    bool allMembersInSameSection() { return inSameSection; }
+    void writeTagFile(FTextStream &);
+    void addGroupedInheritedMembers(OutputList &ol,ClassDef *cd,
+               MemberListType lt,
+               ClassDef *inheritedFrom,const QCString &inheritId);
+
+    const QCString &documentation() const { return doc; }
+    bool allMembersInSameSection() const { return inSameSection; }
     void addToDeclarationSection();
-    int countDecMembers(/*bool sectionPerType*/);
+    int countDecMembers(GroupDef *gd=0);
+    int countDocMembers();
+    int countGroupedInheritedMembers(MemberListType lt);
     void distributeMemberGroupDocumentation();
     void findSectionsInDocumentation();
     int varCount() const;
@@ -68,29 +81,43 @@ class MemberGroup
     int friendCount() const;
     int numDecMembers() const;
     int numDocMembers() const;
+    int countInheritableMembers(ClassDef *inheritedFrom) const;
     void setInGroup(bool b);
     void addListReferences(Definition *d);
+    void setRefItems(const QList<ListItemInfo> *sli);
     MemberList *members() const { return memberList; }
     Definition *parent() const { return m_parent; }
+    QCString anchor() const;
+
+    QCString docFile() const { return m_docFile; }
+    int docLine() const { return m_docLine; }
+
+    void marshal(StorageIntf *s);
+    void unmarshal(StorageIntf *s);
 
   private: 
     MemberList *memberList;      // list of all members in the group
+    MemberList *inDeclSection;
     int grpId;
     QCString grpHeader;
     QCString fileName;           // base name of the generated file
     Definition *scope;
     QCString doc;
-    MemberList *inDeclSection;
     bool inSameSection;
     int  m_numDecMembers;
+    int  m_numDocMembers;
     Definition *m_parent;
     QCString m_docFile;
+    int m_docLine;
+    QList<ListItemInfo> *m_xrefListItems;
 };
 
+/** A list of MemberGroup objects. */
 class MemberGroupList : public QList<MemberGroup>
 {
 };
 
+/** An iterator for MemberGroup objects in a MemberGroupList. */
 class MemberGroupListIterator : public QListIterator<MemberGroup>
 {
   public:
@@ -98,33 +125,31 @@ class MemberGroupListIterator : public QListIterator<MemberGroup>
       QListIterator<MemberGroup>(l) {}
 };
 
+/** A sorted dictionary of MemberGroup objects. */
 class MemberGroupSDict : public SIntDict<MemberGroup>
 {
   public:
     MemberGroupSDict(int size=17) : SIntDict<MemberGroup>(size) {}
    ~MemberGroupSDict() {}
+ private:
+    int compareValues(const MemberGroup *item1,const MemberGroup *item2) const
+    {
+      return item1->groupId() - item2->groupId();
+    }
 };
 
+/** Data collected for a member group */
 struct MemberGroupInfo
 {
+  MemberGroupInfo() : docLine(-1), m_sli(0) {}
+ ~MemberGroupInfo() { delete m_sli; m_sli=0; }
+  void setRefItems(const QList<ListItemInfo> *sli);
   QCString header;
   QCString doc;
   QCString docFile;
+  int docLine;
+  QCString compoundName;
+  QList<ListItemInfo> *m_sli;
 };
 
-//class MemberGroupDict : public QIntDict<MemberGroup>
-//{
-//  public:
-//    MemberGroupDict(int size) : QIntDict<MemberGroup>(size) {}
-//   ~MemberGroupDict() {}
-//};
-
-//class MemberGroupDictIterator : public QIntDictIterator<MemberGroup>
-//{
-//  public:
-//    MemberGroupDictIterator(const MemberGroupDict &d) : 
-//      QIntDictIterator<MemberGroup>(d) {}
-//   ~MemberGroupDictIterator() {}
-//};
-
 #endif
