@@ -1,8 +1,8 @@
 /******************************************************************************
  *
- * $Id: memberlist.h,v 1.19 2001/03/19 19:27:41 root Exp $
+ * 
  *
- * Copyright (C) 1997-2005 by Dimitri van Heesch.
+ * Copyright (C) 1997-2015 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation under the terms of the GNU General Public License is hereby 
@@ -21,20 +21,35 @@
 #include <qlist.h>
 #include "memberdef.h"
 #include "sortdict.h"
+#include "types.h"
 
 class GroupDef;
 class MemberGroup;
 class MemberGroupList;
+class StorageIntf;
 
-class MemberList : public QList<MemberDef> 
-{ 
+/** A list of MemberDef objects. */
+class MemberList : private QList<MemberDef>
+{
+    friend class MemberListIterator;
   public:
     MemberList();
+    MemberList(MemberListType lt);
    ~MemberList();
-    bool insert(uint index,const MemberDef *md);
+    MemberListType listType() const { return m_listType; }
+    static QCString listTypeAsString(MemberListType type);
+
+    /* ---- standard QList methods ---- */
     void inSort(const MemberDef *md);
     void append(const MemberDef *md);
-    int compareItems(GCI item1,GCI item2);
+    void remove(const MemberDef *md);
+    void sort();
+    uint count() const;
+    int findRef(const MemberDef *md) const;
+    MemberDef *getFirst() const;
+    MemberDef *take(uint index);
+
+
     int varCount() const       { ASSERT(m_numDecMembers!=-1); return m_varCnt;     }
     int funcCount() const      { ASSERT(m_numDecMembers!=-1); return m_funcCnt;    }
     int enumCount() const      { ASSERT(m_numDecMembers!=-1); return m_enumCnt;    }
@@ -45,37 +60,66 @@ class MemberList : public QList<MemberDef>
     int friendCount() const    { ASSERT(m_numDecMembers!=-1); return m_friendCnt;  }
     int numDecMembers() const  { ASSERT(m_numDecMembers!=-1); return m_numDecMembers; }
     int numDocMembers() const  { ASSERT(m_numDocMembers!=-1); return m_numDocMembers; }
-    void countDecMembers(/*bool inGroup,bool countSubGroups,bool sectionPerType*/);
-    void countDocMembers();
+    bool needsSorting() const  { return m_needsSorting; }
+    void countDecMembers(bool countEnumValues=FALSE,GroupDef *gd=0);
+    void countDocMembers(bool countEnumValues=FALSE);
+    int countInheritableMembers(ClassDef *inheritedFrom) const;
     void writePlainDeclarations(OutputList &ol,
-               ClassDef *cd,NamespaceDef *nd,FileDef *fd,GroupDef *gd);
+               ClassDef *cd,NamespaceDef *nd,FileDef *fd, GroupDef *gd,
+               ClassDef *inheritedFrom,const char *inheritId);
     void writeDeclarations(OutputList &ol,
                ClassDef *cd,NamespaceDef *nd,FileDef *fd,GroupDef *gd,
-               const char *title,const char *subtitle);
+               const char *title,const char *subtitle,
+               bool showEnumValues=FALSE,bool showInline=FALSE,
+               ClassDef *inheritedFrom=0,MemberListType lt=MemberListType_pubMethods);
     void writeDocumentation(OutputList &ol,const char *scopeName,
-               Definition *container,const char *title);
+               Definition *container,const char *title,bool showEnumValues=FALSE,bool showInline=FALSE);
+    void writeSimpleDocumentation(OutputList &ol,Definition *container);
+    void writeDocumentationPage(OutputList &ol,
+               const char *scopeName, Definition *container);
+    void writeTagFile(FTextStream &);
+    bool declVisible() const;
     void addMemberGroup(MemberGroup *mg);
-    void setInGroup(bool group) { m_inGroup=group; }
+    void setInGroup(bool inGroup) { m_inGroup=inGroup; }
+    void setInFile(bool inFile) { m_inFile=inFile; }
     void addListReferences(Definition *def);
     void findSectionsInDocumentation();
+    void setNeedsSorting(bool b);
     MemberGroupList *getMemberGroupList() const { return memberGroupList; }
 
+    void marshal(StorageIntf *s);
+    void unmarshal(StorageIntf *s);
+
   private:
-    int m_varCnt,m_funcCnt,m_enumCnt,m_enumValCnt,m_typeCnt;
-    int m_protoCnt,m_defCnt,m_friendCnt; 
+    int compareValues(const MemberDef *item1,const MemberDef *item2) const;
+    int countEnumValues(MemberDef *md,bool setAnonEnumType) const;
+    int m_varCnt;
+    int m_funcCnt;
+    int m_enumCnt;
+    int m_enumValCnt;
+    int m_typeCnt;
+    int m_protoCnt;
+    int m_defCnt;
+    int m_friendCnt; 
     int m_numDecMembers; // number of members in the brief part of the memberlist
     int m_numDocMembers; // number of members in the detailed part of the memberlist
     MemberGroupList *memberGroupList;
-    bool m_inGroup; // is this list part of a group
+    bool m_inGroup; // is this list part of a group definition
+    bool m_inFile;  // is this list part of a file definition
+    MemberListType m_listType;
+    bool m_needsSorting;
+    QDict<int> m_overloadCount;
 };
 
+/** An iterator for MemberDef objects in a MemberList. */
 class MemberListIterator : public QListIterator<MemberDef>
 {
   public:
-    MemberListIterator(const QList<MemberDef> &list);
+    MemberListIterator(const MemberList &list);
     virtual ~MemberListIterator() {}
 };
 
+/** An unsorted dictionary of MemberDef objects. */
 class MemberDict : public QDict<MemberDef>
 {
   public:
@@ -83,12 +127,14 @@ class MemberDict : public QDict<MemberDef>
     virtual ~MemberDict() {}
 };
 
+/** A sorted dictionary of MemberDef objects. */
 class MemberSDict : public SDict<MemberDef>
 {
   public:
     MemberSDict(int size=17) : SDict<MemberDef>(size) {}
     virtual ~MemberSDict() {}
-    int compareItems(GCI item1,GCI item2);
+  private:
+    int compareValues(const MemberDef *item1,const MemberDef *item2) const;
 };
 
 
