@@ -1,12 +1,12 @@
 /******************************************************************************
  *
- * 
  *
- * Copyright (C) 1997-2014 by Dimitri van Heesch.
+ *
+ * Copyright (C) 1997-2015 by Dimitri van Heesch.
  *
  * Permission to use, copy, modify, and distribute this software and its
- * documentation under the terms of the GNU General Public License is hereby 
- * granted. No representations are made about the suitability of this software 
+ * documentation under the terms of the GNU General Public License is hereby
+ * granted. No representations are made about the suitability of this software
  * for any purpose. It is provided "as is" without express or implied warranty.
  * See the GNU General Public License for more details.
  *
@@ -41,10 +41,10 @@
 // file format: (all multi-byte values are stored in big endian format)
 //   4 byte header
 //   256*256*4 byte index (4 bytes)
-//   for each index entry: a zero terminated list of words 
+//   for each index entry: a zero terminated list of words
 //   for each word: a \0 terminated string + 4 byte offset to the stats info
 //   padding bytes to align at 4 byte boundary
-//   for each word: the number of urls (4 bytes) 
+//   for each word: the number of urls (4 bytes)
 //               + for each url containing the word 8 bytes statistics
 //                 (4 bytes index to url string + 4 bytes frequency counter)
 //   for each url: a \0 terminated string
@@ -75,7 +75,7 @@ void IndexWord::addUrlIndex(int idx,bool hiPriority)
 
 //--------------------------------------------------------------------
 
-SearchIndex::SearchIndex() : SearchIndexIntf(Internal), 
+SearchIndex::SearchIndex() : SearchIndexIntf(Internal),
       m_words(328829), m_index(numIndexEntries), m_url2IdMap(10007), m_urls(10007), m_urlIndex(-1)
 {
   int i;
@@ -92,15 +92,15 @@ void SearchIndex::setCurrentDoc(Definition *ctx,const char *anchor,bool isSource
   assert(!isSourceFile || ctx->definitionType()==Definition::TypeFile);
   //printf("SearchIndex::setCurrentDoc(%s,%s,%s)\n",name,baseName,anchor);
   QCString url=isSourceFile ? ((FileDef*)ctx)->getSourceFileBase() : ctx->getOutputFileBase();
-  url+=Config_getString("HTML_FILE_EXTENSION");
+  url+=Config_getString(HTML_FILE_EXTENSION);
   QCString baseUrl = url;
-  if (anchor) url+=QCString("#")+anchor;  
+  if (anchor) url+=QCString("#")+anchor;
   if (!isSourceFile) baseUrl=url;
   QCString name=ctx->qualifiedName();
   if (ctx->definitionType()==Definition::TypeMember)
   {
     MemberDef *md = (MemberDef *)ctx;
-    name.prepend((md->getLanguage()==SrcLangExt_Fortran  ? 
+    name.prepend((md->getLanguage()==SrcLangExt_Fortran  ?
                  theTranslator->trSubprogram(TRUE,TRUE) :
                  theTranslator->trMember(TRUE,TRUE))+" ");
   }
@@ -188,7 +188,7 @@ static int charsToIndex(const char *word)
   //register ushort h=0;
   //const char *k = word;
   //ushort mask=0xfc00;
-  //while ( *k ) 
+  //while ( *k )
   //{
   //  h = (h&mask)^(h<<6)^(*k++);
   //}
@@ -275,7 +275,7 @@ void SearchIndex::write(const char *fileName)
       IndexWord *iw;
       for (iwi.toFirst();(iw=iwi.current());++iwi)
       {
-        int ws = iw->word().length()+1; 
+        int ws = iw->word().length()+1;
         size+=ws+4; // word + url info list offset
       }
       size+=1; // zero list terminator
@@ -295,8 +295,8 @@ void SearchIndex::write(const char *fileName)
       IndexWord *iw;
       for (iwi.toFirst();(iw=iwi.current());++iwi)
       {
-        offset+= iw->word().length()+1; 
-        offset+=4; // word + offset to url info array 
+        offset+= iw->word().length()+1;
+        offset+=4; // word + offset to url info array
       }
       offset+=1; // zero list terminator
     }
@@ -310,10 +310,9 @@ void SearchIndex::write(const char *fileName)
   padding = size - padding;
 
   //int statsOffset = size;
-  QDictIterator<IndexWord> wdi(m_words);
   //IndexWord *iw;
   int *wordStatOffsets = new int[m_words.count()];
-  
+
   int count=0;
 
   // third pass: compute offset to stats info for each word
@@ -418,7 +417,7 @@ struct SearchDocEntry
   QCString name;
   QCString args;
   QCString extId;
-  QCString url; 
+  QCString url;
   GrowBuf  importantText;
   GrowBuf  normalText;
 };
@@ -477,7 +476,7 @@ static QCString definitionToName(Definition *ctx)
   {
     switch(ctx->definitionType())
     {
-      case Definition::TypeClass: 
+      case Definition::TypeClass:
         return ((ClassDef*)ctx)->compoundTypeString();
       case Definition::TypeFile:
         return "file";
@@ -500,7 +499,7 @@ static QCString definitionToName(Definition *ctx)
 
 void SearchIndexExternal::setCurrentDoc(Definition *ctx,const char *anchor,bool isSourceFile)
 {
-  QCString extId = stripPath(Config_getString("EXTERNAL_SEARCH_ID"));
+  QCString extId = stripPath(Config_getString(EXTERNAL_SEARCH_ID));
   QCString baseName = isSourceFile ? ((FileDef*)ctx)->getSourceFileBase() : ctx->getOutputFileBase();
   QCString url = baseName + Doxygen::htmlFileExtension;
   if (anchor) url+=QCString("#")+anchor;
@@ -584,78 +583,17 @@ void SearchIndexExternal::write(const char *fileName)
 #include "doxygen.h"
 #include "message.h"
 
-//static const char search_script[]=
-//#include "search.js.h"
-//;
+static SearchIndexInfo g_searchIndexInfo[NUM_SEARCH_INDICES];
 
-#define SEARCH_INDEX_ALL         0
-#define SEARCH_INDEX_CLASSES     1
-#define SEARCH_INDEX_NAMESPACES  2
-#define SEARCH_INDEX_FILES       3
-#define SEARCH_INDEX_FUNCTIONS   4
-#define SEARCH_INDEX_VARIABLES   5
-#define SEARCH_INDEX_TYPEDEFS    6
-#define SEARCH_INDEX_ENUMS       7
-#define SEARCH_INDEX_ENUMVALUES  8
-#define SEARCH_INDEX_PROPERTIES  9
-#define SEARCH_INDEX_EVENTS     10
-#define SEARCH_INDEX_RELATED    11
-#define SEARCH_INDEX_DEFINES    12
-#define SEARCH_INDEX_GROUPS     13
-#define SEARCH_INDEX_PAGES      14
-#define NUM_SEARCH_INDICES      15
-
-class SearchDefinitionList : public QList<Definition>
+static void addMemberToSearchIndex(MemberDef *md)
 {
-  public:
-    SearchDefinitionList(uint letter) : m_letter(letter) {}
-    uint letter() const { return m_letter; }
-  private:
-    uint m_letter;
-};
-
-class SearchIndexList : public SDict< SearchDefinitionList >
-{
-  public:
-    typedef Definition ElementType;
-    SearchIndexList(uint letter) : SDict<SearchDefinitionList>(17,FALSE), m_letter(letter)
-    {
-      setAutoDelete(TRUE);
-    }
-   ~SearchIndexList() {}
-    void append(Definition *d)
-    {
-      SearchDefinitionList *l = find(d->name());
-      if (l==0)
-      {
-        l=new SearchDefinitionList(m_letter);
-        SDict<SearchDefinitionList>::append(d->name(),l);
-      }
-      l->append(d);
-    }
-    uint letter() const { return m_letter; }
-  private:
-    int compareValues(const SearchDefinitionList *md1, const SearchDefinitionList *md2) const
-    {
-      QCString n1 = md1->getFirst()->localName();
-      QCString n2 = md2->getFirst()->localName();
-      return qstricmp(n1.data(),n2.data());
-    }
-    uint m_letter;
-};
-
-static void addMemberToSearchIndex(
-         LetterToIndexMap<SearchIndexList> symbols[NUM_SEARCH_INDICES],
-         int symbolCount[NUM_SEARCH_INDICES],
-         MemberDef *md)
-{
-  static bool hideFriendCompounds = Config_getBool("HIDE_FRIEND_COMPOUNDS");
+  static bool hideFriendCompounds = Config_getBool(HIDE_FRIEND_COMPOUNDS);
   bool isLinkable = md->isLinkable();
   ClassDef *cd=0;
   NamespaceDef *nd=0;
   FileDef *fd=0;
   GroupDef *gd=0;
-  if (isLinkable && 
+  if (isLinkable &&
       (
        ((cd=md->getClassDef()) && cd->isLinkable() && cd->templateMaster()==0) ||
        ((gd=md->getGroupDef()) && gd->isLinkable())
@@ -663,103 +601,87 @@ static void addMemberToSearchIndex(
      )
   {
     QCString n = md->name();
-    if (!n.isEmpty()) 
+    if (!n.isEmpty())
     {
       uint letter = getUtf8CodeToLower(n,0);
       bool isFriendToHide = hideFriendCompounds &&
-        (QCString(md->typeString())=="friend class" || 
+        (QCString(md->typeString())=="friend class" ||
          QCString(md->typeString())=="friend struct" ||
          QCString(md->typeString())=="friend union");
       if (!(md->isFriend() && isFriendToHide))
       {
-        symbols[SEARCH_INDEX_ALL].append(letter,md);
-        symbolCount[SEARCH_INDEX_ALL]++;
+        g_searchIndexInfo[SEARCH_INDEX_ALL].symbolList.append(letter,md);
       }
       if (md->isFunction() || md->isSlot() || md->isSignal())
       {
-        symbols[SEARCH_INDEX_FUNCTIONS].append(letter,md);
-        symbolCount[SEARCH_INDEX_FUNCTIONS]++;
-      } 
+        g_searchIndexInfo[SEARCH_INDEX_FUNCTIONS].symbolList.append(letter,md);
+      }
       else if (md->isVariable())
       {
-        symbols[SEARCH_INDEX_VARIABLES].append(letter,md);
-        symbolCount[SEARCH_INDEX_VARIABLES]++;
+        g_searchIndexInfo[SEARCH_INDEX_VARIABLES].symbolList.append(letter,md);
       }
       else if (md->isTypedef())
       {
-        symbols[SEARCH_INDEX_TYPEDEFS].append(letter,md);
-        symbolCount[SEARCH_INDEX_TYPEDEFS]++;
+        g_searchIndexInfo[SEARCH_INDEX_TYPEDEFS].symbolList.append(letter,md);
       }
       else if (md->isEnumerate())
       {
-        symbols[SEARCH_INDEX_ENUMS].append(letter,md);
-        symbolCount[SEARCH_INDEX_ENUMS]++;
+        g_searchIndexInfo[SEARCH_INDEX_ENUMS].symbolList.append(letter,md);
       }
       else if (md->isEnumValue())
       {
-        symbols[SEARCH_INDEX_ENUMVALUES].append(letter,md);
-        symbolCount[SEARCH_INDEX_ENUMVALUES]++;
+        g_searchIndexInfo[SEARCH_INDEX_ENUMVALUES].symbolList.append(letter,md);
       }
       else if (md->isProperty())
       {
-        symbols[SEARCH_INDEX_PROPERTIES].append(letter,md);
-        symbolCount[SEARCH_INDEX_PROPERTIES]++;
+        g_searchIndexInfo[SEARCH_INDEX_PROPERTIES].symbolList.append(letter,md);
       }
       else if (md->isEvent())
       {
-        symbols[SEARCH_INDEX_EVENTS].append(letter,md);
-        symbolCount[SEARCH_INDEX_EVENTS]++;
+        g_searchIndexInfo[SEARCH_INDEX_EVENTS].symbolList.append(letter,md);
       }
       else if (md->isRelated() || md->isForeign() ||
                (md->isFriend() && !isFriendToHide))
       {
-        symbols[SEARCH_INDEX_RELATED].append(letter,md);
-        symbolCount[SEARCH_INDEX_RELATED]++;
+        g_searchIndexInfo[SEARCH_INDEX_RELATED].symbolList.append(letter,md);
       }
     }
   }
-  else if (isLinkable && 
-      (((nd=md->getNamespaceDef()) && nd->isLinkable()) || 
+  else if (isLinkable &&
+      (((nd=md->getNamespaceDef()) && nd->isLinkable()) ||
        ((fd=md->getFileDef())      && fd->isLinkable())
       )
      )
   {
     QCString n = md->name();
-    if (!n.isEmpty()) 
+    if (!n.isEmpty())
     {
       uint letter = getUtf8CodeToLower(n,0);
-      symbols[SEARCH_INDEX_ALL].append(letter,md);
-      symbolCount[SEARCH_INDEX_ALL]++;
+      g_searchIndexInfo[SEARCH_INDEX_ALL].symbolList.append(letter,md);
 
-      if (md->isFunction()) 
+      if (md->isFunction())
       {
-        symbols[SEARCH_INDEX_FUNCTIONS].append(letter,md);
-        symbolCount[SEARCH_INDEX_FUNCTIONS]++;
+        g_searchIndexInfo[SEARCH_INDEX_FUNCTIONS].symbolList.append(letter,md);
       }
-      else if (md->isVariable()) 
+      else if (md->isVariable())
       {
-        symbols[SEARCH_INDEX_VARIABLES].append(letter,md);
-        symbolCount[SEARCH_INDEX_VARIABLES]++;
+        g_searchIndexInfo[SEARCH_INDEX_VARIABLES].symbolList.append(letter,md);
       }
       else if (md->isTypedef())
       {
-        symbols[SEARCH_INDEX_TYPEDEFS].append(letter,md);
-        symbolCount[SEARCH_INDEX_TYPEDEFS]++;
+        g_searchIndexInfo[SEARCH_INDEX_TYPEDEFS].symbolList.append(letter,md);
       }
       else if (md->isEnumerate())
       {
-        symbols[SEARCH_INDEX_ENUMS].append(letter,md);
-        symbolCount[SEARCH_INDEX_ENUMS]++;
+        g_searchIndexInfo[SEARCH_INDEX_ENUMS].symbolList.append(letter,md);
       }
       else if (md->isEnumValue())
       {
-        symbols[SEARCH_INDEX_ENUMVALUES].append(letter,md);
-        symbolCount[SEARCH_INDEX_ENUMVALUES]++;
+        g_searchIndexInfo[SEARCH_INDEX_ENUMVALUES].symbolList.append(letter,md);
       }
       else if (md->isDefine())
       {
-        symbols[SEARCH_INDEX_DEFINES].append(letter,md);
-        symbolCount[SEARCH_INDEX_DEFINES]++;
+        g_searchIndexInfo[SEARCH_INDEX_DEFINES].symbolList.append(letter,md);
       }
     }
   }
@@ -793,55 +715,43 @@ static QCString searchId(const QCString &s)
   return result;
 }
 
-static int g_searchIndexCount[NUM_SEARCH_INDICES];
-static LetterToIndexMap<SearchIndexList> g_searchIndexSymbols[NUM_SEARCH_INDICES];
-static const char *g_searchIndexName[NUM_SEARCH_INDICES] = 
-{ 
-    "all",
-    "classes",
-    "namespaces",
-    "files",
-    "functions",
-    "variables",
-    "typedefs", 
-    "enums", 
-    "enumvalues",
-    "properties", 
-    "events", 
-    "related",
-    "defines",
-    "groups",
-    "pages"
-};
-
-
-class SearchIndexCategoryMapping
+void createJavascriptSearchIndex()
 {
-  public:
-    SearchIndexCategoryMapping()
-    {
-      categoryLabel[SEARCH_INDEX_ALL]        = theTranslator->trAll();
-      categoryLabel[SEARCH_INDEX_CLASSES]    = theTranslator->trClasses();
-      categoryLabel[SEARCH_INDEX_NAMESPACES] = theTranslator->trNamespace(TRUE,FALSE);
-      categoryLabel[SEARCH_INDEX_FILES]      = theTranslator->trFile(TRUE,FALSE);
-      categoryLabel[SEARCH_INDEX_FUNCTIONS]  = theTranslator->trFunctions();
-      categoryLabel[SEARCH_INDEX_VARIABLES]  = theTranslator->trVariables();
-      categoryLabel[SEARCH_INDEX_TYPEDEFS]   = theTranslator->trTypedefs();
-      categoryLabel[SEARCH_INDEX_ENUMS]      = theTranslator->trEnumerations();
-      categoryLabel[SEARCH_INDEX_ENUMVALUES] = theTranslator->trEnumerationValues();
-      categoryLabel[SEARCH_INDEX_PROPERTIES] = theTranslator->trProperties();
-      categoryLabel[SEARCH_INDEX_EVENTS]     = theTranslator->trEvents();
-      categoryLabel[SEARCH_INDEX_RELATED]    = theTranslator->trFriends();
-      categoryLabel[SEARCH_INDEX_DEFINES]    = theTranslator->trDefines();
-      categoryLabel[SEARCH_INDEX_GROUPS]     = theTranslator->trGroup(TRUE,FALSE);
-      categoryLabel[SEARCH_INDEX_PAGES]      = theTranslator->trPage(TRUE,FALSE);
-    }
-    QCString categoryLabel[NUM_SEARCH_INDICES];
-};
+  // set index names
+  g_searchIndexInfo[SEARCH_INDEX_ALL].name        = "all";
+  g_searchIndexInfo[SEARCH_INDEX_CLASSES].name    = "classes";
+  g_searchIndexInfo[SEARCH_INDEX_NAMESPACES].name = "namespaces";
+  g_searchIndexInfo[SEARCH_INDEX_FILES].name      = "files";
+  g_searchIndexInfo[SEARCH_INDEX_FUNCTIONS].name  = "functions";
+  g_searchIndexInfo[SEARCH_INDEX_VARIABLES].name  = "variables";
+  g_searchIndexInfo[SEARCH_INDEX_TYPEDEFS].name   = "typedefs";
+  g_searchIndexInfo[SEARCH_INDEX_ENUMS].name      = "enums";
+  g_searchIndexInfo[SEARCH_INDEX_ENUMVALUES].name = "enumvalues";
+  g_searchIndexInfo[SEARCH_INDEX_PROPERTIES].name = "properties";
+  g_searchIndexInfo[SEARCH_INDEX_EVENTS].name     = "events";
+  g_searchIndexInfo[SEARCH_INDEX_RELATED].name    = "related";
+  g_searchIndexInfo[SEARCH_INDEX_DEFINES].name    = "defines";
+  g_searchIndexInfo[SEARCH_INDEX_GROUPS].name     = "groups";
+  g_searchIndexInfo[SEARCH_INDEX_PAGES].name      = "pages";
 
-void writeJavascriptSearchIndex()
-{
-  if (!Config_getBool("GENERATE_HTML")) return;
+  // set index texts
+  g_searchIndexInfo[SEARCH_INDEX_ALL].text        = theTranslator->trAll();
+  g_searchIndexInfo[SEARCH_INDEX_CLASSES].text    = theTranslator->trClasses();
+  g_searchIndexInfo[SEARCH_INDEX_NAMESPACES].text = theTranslator->trNamespace(TRUE,FALSE);
+  g_searchIndexInfo[SEARCH_INDEX_FILES].text      = theTranslator->trFile(TRUE,FALSE);
+  g_searchIndexInfo[SEARCH_INDEX_FUNCTIONS].text  = theTranslator->trFunctions();
+  g_searchIndexInfo[SEARCH_INDEX_VARIABLES].text  = theTranslator->trVariables();
+  g_searchIndexInfo[SEARCH_INDEX_TYPEDEFS].text   = theTranslator->trTypedefs();
+  g_searchIndexInfo[SEARCH_INDEX_ENUMS].text      = theTranslator->trEnumerations();
+  g_searchIndexInfo[SEARCH_INDEX_ENUMVALUES].text = theTranslator->trEnumerationValues();
+  g_searchIndexInfo[SEARCH_INDEX_PROPERTIES].text = theTranslator->trProperties();
+  g_searchIndexInfo[SEARCH_INDEX_EVENTS].text     = theTranslator->trEvents();
+  g_searchIndexInfo[SEARCH_INDEX_RELATED].text    = theTranslator->trFriends();
+  g_searchIndexInfo[SEARCH_INDEX_DEFINES].text    = theTranslator->trDefines();
+  g_searchIndexInfo[SEARCH_INDEX_GROUPS].text     = theTranslator->trGroup(TRUE,FALSE);
+  g_searchIndexInfo[SEARCH_INDEX_PAGES].text      = theTranslator->trPage(TRUE,FALSE);
+
+  // add symbols to letter -> symbol list map
 
   // index classes
   ClassSDict::Iterator cli(*Doxygen::classSDict);
@@ -851,10 +761,8 @@ void writeJavascriptSearchIndex()
     uint letter = getUtf8CodeToLower(cd->localName(),0);
     if (cd->isLinkable() && isId(letter))
     {
-      g_searchIndexSymbols[SEARCH_INDEX_ALL].append(letter,cd);
-      g_searchIndexSymbols[SEARCH_INDEX_CLASSES].append(letter,cd);
-      g_searchIndexCount[SEARCH_INDEX_ALL]++;
-      g_searchIndexCount[SEARCH_INDEX_CLASSES]++;
+      g_searchIndexInfo[SEARCH_INDEX_ALL].symbolList.append(letter,cd);
+      g_searchIndexInfo[SEARCH_INDEX_CLASSES].symbolList.append(letter,cd);
     }
   }
 
@@ -866,10 +774,8 @@ void writeJavascriptSearchIndex()
     uint letter = getUtf8CodeToLower(nd->name(),0);
     if (nd->isLinkable() && isId(letter))
     {
-      g_searchIndexSymbols[SEARCH_INDEX_ALL].append(letter,nd);
-      g_searchIndexSymbols[SEARCH_INDEX_NAMESPACES].append(letter,nd);
-      g_searchIndexCount[SEARCH_INDEX_ALL]++;
-      g_searchIndexCount[SEARCH_INDEX_NAMESPACES]++;
+      g_searchIndexInfo[SEARCH_INDEX_ALL].symbolList.append(letter,nd);
+      g_searchIndexInfo[SEARCH_INDEX_NAMESPACES].symbolList.append(letter,nd);
     }
   }
 
@@ -885,10 +791,8 @@ void writeJavascriptSearchIndex()
       uint letter = getUtf8CodeToLower(fd->name(),0);
       if (fd->isLinkable() && isId(letter))
       {
-        g_searchIndexSymbols[SEARCH_INDEX_ALL].append(letter,fd);
-        g_searchIndexSymbols[SEARCH_INDEX_FILES].append(letter,fd);
-        g_searchIndexCount[SEARCH_INDEX_ALL]++;
-        g_searchIndexCount[SEARCH_INDEX_FILES]++;
+        g_searchIndexInfo[SEARCH_INDEX_ALL].symbolList.append(letter,fd);
+        g_searchIndexInfo[SEARCH_INDEX_FILES].symbolList.append(letter,fd);
       }
     }
   }
@@ -905,7 +809,7 @@ void writeJavascriptSearchIndex()
       // for each member definition
       for (mni.toFirst();(md=mni.current());++mni)
       {
-        addMemberToSearchIndex(g_searchIndexSymbols,g_searchIndexCount,md);
+        addMemberToSearchIndex(md);
       }
     }
   }
@@ -922,7 +826,7 @@ void writeJavascriptSearchIndex()
       // for each member definition
       for (mni.toFirst();(md=mni.current());++mni)
       {
-        addMemberToSearchIndex(g_searchIndexSymbols,g_searchIndexCount,md);
+        addMemberToSearchIndex(md);
       }
     }
   }
@@ -941,10 +845,8 @@ void writeJavascriptSearchIndex()
         uint letter = charCode<128 ? tolower(charCode) : charCode;
         if (isId(letter))
         {
-          g_searchIndexSymbols[SEARCH_INDEX_ALL].append(letter,gd);
-          g_searchIndexSymbols[SEARCH_INDEX_GROUPS].append(letter,gd);
-          g_searchIndexCount[SEARCH_INDEX_ALL]++;
-          g_searchIndexCount[SEARCH_INDEX_GROUPS]++;
+          g_searchIndexInfo[SEARCH_INDEX_ALL].symbolList.append(letter,gd);
+          g_searchIndexInfo[SEARCH_INDEX_GROUPS].symbolList.append(letter,gd);
         }
       }
     }
@@ -964,10 +866,8 @@ void writeJavascriptSearchIndex()
         uint letter = charCode<128 ? tolower(charCode) : charCode;
         if (isId(letter))
         {
-          g_searchIndexSymbols[SEARCH_INDEX_ALL].append(letter,pd);
-          g_searchIndexSymbols[SEARCH_INDEX_PAGES].append(letter,pd);
-          g_searchIndexCount[SEARCH_INDEX_ALL]++;
-          g_searchIndexCount[SEARCH_INDEX_PAGES]++;
+          g_searchIndexInfo[SEARCH_INDEX_ALL].symbolList.append(letter,pd);
+          g_searchIndexInfo[SEARCH_INDEX_PAGES].symbolList.append(letter,pd);
         }
       }
     }
@@ -981,38 +881,40 @@ void writeJavascriptSearchIndex()
       uint letter = charCode<128 ? tolower(charCode) : charCode;
       if (isId(letter))
       {
-        g_searchIndexSymbols[SEARCH_INDEX_ALL].append(letter,Doxygen::mainPage);
-        g_searchIndexSymbols[SEARCH_INDEX_PAGES].append(letter,Doxygen::mainPage);
-        g_searchIndexCount[SEARCH_INDEX_ALL]++;
-        g_searchIndexCount[SEARCH_INDEX_PAGES]++;
+        g_searchIndexInfo[SEARCH_INDEX_ALL].symbolList.append(letter,Doxygen::mainPage);
+        g_searchIndexInfo[SEARCH_INDEX_PAGES].symbolList.append(letter,Doxygen::mainPage);
       }
     }
   }
-  
+
   // sort all lists
   int i;
   for (i=0;i<NUM_SEARCH_INDICES;i++)
   {
-    SIntDict<SearchIndexList>::Iterator it(g_searchIndexSymbols[i]);
+    SIntDict<SearchIndexList>::Iterator it(g_searchIndexInfo[i].symbolList);
     SearchIndexList *sl;
     for (it.toFirst();(sl=it.current());++it)
     {
       sl->sort();
     }
   }
+}
 
+void writeJavascriptSearchIndex()
+{
+  int i;
   // write index files
-  QCString searchDirName = Config_getString("HTML_OUTPUT")+"/search";
+  QCString searchDirName = Config_getString(HTML_OUTPUT)+"/search";
 
   for (i=0;i<NUM_SEARCH_INDICES;i++) // for each index
   {
-    SIntDict<SearchIndexList>::Iterator it(g_searchIndexSymbols[i]);
+    SIntDict<SearchIndexList>::Iterator it(g_searchIndexInfo[i].symbolList);
     SearchIndexList *sl;
     int p=0;
     for (it.toFirst();(sl=it.current());++it,++p) // for each letter
     {
       QCString baseName;
-      baseName.sprintf("%s_%x",g_searchIndexName[i],p);
+      baseName.sprintf("%s_%x",g_searchIndexInfo[i].name.data(),p);
 
       QCString fileName = searchDirName + "/"+baseName+".html";
       QCString dataFileName = searchDirName + "/"+baseName+".js";
@@ -1038,18 +940,22 @@ void writeJavascriptSearchIndex()
           t << "<div class=\"SRStatus\" id=\"Loading\">" << theTranslator->trLoading() << "</div>" << endl;
           t << "<div id=\"SRResults\"></div>" << endl; // here the results will be inserted
           t << "<script type=\"text/javascript\"><!--" << endl;
+					t << "/* @license magnet:?xt=urn:btih:cf05388f2679ee054f2beb29a391d25f4e673ac3&amp;dn=gpl-2.0.txt GPL-v2 */\n";
           t << "createResults();" << endl; // this function will insert the results
+					t << "/* @license-end */\n";
           t << "--></script>" << endl;
-          t << "<div class=\"SRStatus\" id=\"Searching\">" 
+          t << "<div class=\"SRStatus\" id=\"Searching\">"
             << theTranslator->trSearching() << "</div>" << endl;
           t << "<div class=\"SRStatus\" id=\"NoMatches\">"
             << theTranslator->trNoMatches() << "</div>" << endl;
 
           t << "<script type=\"text/javascript\"><!--" << endl;
+					t << "/* @license magnet:?xt=urn:btih:cf05388f2679ee054f2beb29a391d25f4e673ac3&amp;dn=gpl-2.0.txt GPL-v2 */\n";
           t << "document.getElementById(\"Loading\").style.display=\"none\";" << endl;
           t << "document.getElementById(\"NoMatches\").style.display=\"none\";" << endl;
           t << "var searchResults = new SearchResults(\"searchResults\");" << endl;
           t << "searchResults.Search();" << endl;
+					t << "/* @license-end */\n";
           t << "--></script>" << endl;
           t << "</div>" << endl; // SRIndex
           t << "</body>" << endl;
@@ -1077,7 +983,6 @@ void writeJavascriptSearchIndex()
         for (li.toFirst();(dl=li.current());++li)
         {
           Definition *d = dl->getFirst();
-          QCString id = d->localName();
 
           if (!firstEntry)
           {
@@ -1085,17 +990,7 @@ void writeJavascriptSearchIndex()
           }
           firstEntry=FALSE;
 
-          QCString dispName = d->localName();
-          if (d->definitionType()==Definition::TypeGroup)
-          {
-            dispName = ((GroupDef*)d)->groupTitle();
-          }
-          else if (d->definitionType()==Definition::TypePage)
-          {
-            dispName = ((PageDef*)d)->title();
-          }
-          ti << "  ['" << searchId(dispName) << "',['" 
-            << convertToXML(dispName) << "',[";
+          ti << "  ['" << dl->id() << "',['" << convertToXML(dl->name()) << "',[";
 
           if (dl->count()==1) // item with a unique name
           {
@@ -1112,7 +1007,7 @@ void writeJavascriptSearchIndex()
             }
             ti << "',";
 
-            static bool extLinksInWindow = Config_getBool("EXT_LINKS_IN_WINDOW");
+            static bool extLinksInWindow = Config_getBool(EXT_LINKS_IN_WINDOW);
             if (!extLinksInWindow || d->getReference().isEmpty())
             {
               ti << "1,";
@@ -1171,7 +1066,7 @@ void writeJavascriptSearchIndex()
               }
               ti << "',";
 
-              static bool extLinksInWindow = Config_getBool("EXT_LINKS_IN_WINDOW");
+              static bool extLinksInWindow = Config_getBool(EXT_LINKS_IN_WINDOW);
               if (!extLinksInWindow || d->getReference().isEmpty())
               {
                 ti << "1,";
@@ -1183,13 +1078,13 @@ void writeJavascriptSearchIndex()
               bool found=FALSE;
               overloadedFunction = ((prevScope!=0 && scope==prevScope) ||
                   (scope && scope==nextScope)
-                  ) && md && 
+                  ) && md &&
                 (md->isFunction() || md->isSlot());
               QCString prefix;
               if (md) prefix=convertToXML(md->localName());
               if (overloadedFunction) // overloaded member function
               {
-                prefix+=convertToXML(md->argsString()); 
+                prefix+=convertToXML(md->argsString());
                 // show argument list to disambiguate overloaded functions
               }
               else if (md) // unique member function
@@ -1221,11 +1116,11 @@ void writeJavascriptSearchIndex()
                   }
                 }
               }
-              else if (md && (md->getClassDef() || md->getNamespaceDef())) 
+              else if (md && (md->getClassDef() || md->getNamespaceDef()))
                 // member in class or namespace scope
               {
                 SrcLangExt lang = md->getLanguage();
-                name = convertToXML(d->getOuterScope()->qualifiedName()) 
+                name = convertToXML(d->getOuterScope()->qualifiedName())
                   + getLanguageSpecificSeparator(lang) + prefix;
                 found = TRUE;
               }
@@ -1276,12 +1171,12 @@ void writeJavascriptSearchIndex()
       int j=0;
       for (i=0;i<NUM_SEARCH_INDICES;i++)
       {
-        if (g_searchIndexCount[i]>0)
+        if (g_searchIndexInfo[i].symbolList.count()>0)
         {
           if (!first) t << "," << endl;
           t << "  " << j << ": \"";
 
-          SIntDict<SearchIndexList>::Iterator it(g_searchIndexSymbols[i]);
+          SIntDict<SearchIndexList>::Iterator it(g_searchIndexInfo[i].symbolList);
           SearchIndexList *sl;
           for (it.toFirst();(sl=it.current());++it) // for each letter
           {
@@ -1300,10 +1195,10 @@ void writeJavascriptSearchIndex()
       j=0;
       for (i=0;i<NUM_SEARCH_INDICES;i++)
       {
-        if (g_searchIndexCount[i]>0)
+        if (g_searchIndexInfo[i].symbolList.count()>0)
         {
           if (!first) t << "," << endl;
-          t << "  " << j << ": \"" << g_searchIndexName[i] << "\"";
+          t << "  " << j << ": \"" << g_searchIndexInfo[i].name << "\"";
           first=FALSE;
           j++;
         }
@@ -1313,14 +1208,13 @@ void writeJavascriptSearchIndex()
       t << "var indexSectionLabels =" << endl;
       t << "{" << endl;
       first=TRUE;
-      static SearchIndexCategoryMapping map;
       j=0;
       for (i=0;i<NUM_SEARCH_INDICES;i++)
       {
-        if (g_searchIndexCount[i]>0)
+        if (g_searchIndexInfo[i].symbolList.count()>0)
         {
           if (!first) t << "," << endl;
-          t << "  " << j << ": \"" << convertToXML(map.categoryLabel[i]) << "\"";
+          t << "  " << j << ": \"" << convertToXML(g_searchIndexInfo[i].text) << "\"";
           first=FALSE;
           j++;
         }
@@ -1354,13 +1248,62 @@ void writeJavascriptSearchIndex()
   Doxygen::indexList->addStyleSheetFile("search/search.js");
 }
 
+const SearchIndexInfo *getSearchIndices()
+{
+  return g_searchIndexInfo;
+}
+
+//---------------------------------------------------------------------------------------------
+
+SearchIndexList::SearchIndexList(uint letter)
+  : SDict< SearchDefinitionList >(17,FALSE), m_letter(letter)
+{
+  setAutoDelete(TRUE);
+}
+
+SearchIndexList::~SearchIndexList()
+{
+}
+
+void SearchIndexList::append(Definition *d)
+{
+  QCString dispName = d->localName();
+  SearchDefinitionList *l = find(dispName);
+  if (l==0)
+  {
+    if (d->definitionType()==Definition::TypeGroup)
+    {
+      dispName = ((GroupDef*)d)->groupTitle();
+    }
+    else if (d->definitionType()==Definition::TypePage)
+    {
+      dispName = ((PageDef*)d)->title();
+    }
+    l=new SearchDefinitionList(searchId(dispName),dispName);
+    SDict< SearchDefinitionList >::append(dispName,l);
+  }
+  l->append(d);
+}
+
+uint SearchIndexList::letter() const
+{
+  return m_letter;
+}
+
+int SearchIndexList::compareValues(const SearchDefinitionList *md1, const SearchDefinitionList *md2) const
+{
+  QCString n1 = md1->getFirst()->localName();
+  QCString n2 = md2->getFirst()->localName();
+  return qstricmp(n1.data(),n2.data());
+}
+
 //---------------------------------------------------------------------------------------------
 
 void initSearchIndexer()
 {
-  static bool searchEngine      = Config_getBool("SEARCHENGINE");
-  static bool serverBasedSearch = Config_getBool("SERVER_BASED_SEARCH");
-  static bool externalSearch    = Config_getBool("EXTERNAL_SEARCH");
+  static bool searchEngine      = Config_getBool(SEARCHENGINE);
+  static bool serverBasedSearch = Config_getBool(SERVER_BASED_SEARCH);
+  static bool externalSearch    = Config_getBool(EXTERNAL_SEARCH);
   if (searchEngine && serverBasedSearch)
   {
     if (externalSearch) // external tools produce search index and engine
@@ -1382,5 +1325,3 @@ void finializeSearchIndexer()
 {
   delete Doxygen::searchIndex;
 }
-
-
